@@ -10,10 +10,11 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import mock
+from unittest import mock
 
 from keystoneauth1 import adapter
 
+from openstack.baremetal.v1 import node as _node
 from openstack.baremetal_introspection.v1 import _proxy
 from openstack.baremetal_introspection.v1 import introspection
 from openstack import exceptions
@@ -21,18 +22,40 @@ from openstack.tests.unit import base
 from openstack.tests.unit import test_proxy_base
 
 
+@mock.patch.object(introspection.Introspection, 'create', autospec=True)
+class TestStartIntrospection(base.TestCase):
+
+    def setUp(self):
+        super(TestStartIntrospection, self).setUp()
+        self.session = mock.Mock(spec=adapter.Adapter)
+        self.proxy = _proxy.Proxy(self.session)
+
+    def test_create_introspection(self, mock_create):
+        self.proxy.start_introspection('abcd')
+        mock_create.assert_called_once_with(mock.ANY, self.proxy)
+        introspect = mock_create.call_args[0][0]
+        self.assertEqual('abcd', introspect.id)
+
+    def test_create_introspection_with_node(self, mock_create):
+        self.proxy.start_introspection(_node.Node(id='abcd'))
+        mock_create.assert_called_once_with(mock.ANY, self.proxy)
+        introspect = mock_create.call_args[0][0]
+        self.assertEqual('abcd', introspect.id)
+
+    def test_create_introspection_manage_boot(self, mock_create):
+        self.proxy.start_introspection('abcd', manage_boot=False)
+        mock_create.assert_called_once_with(mock.ANY, self.proxy,
+                                            manage_boot=False)
+        introspect = mock_create.call_args[0][0]
+        self.assertEqual('abcd', introspect.id)
+
+
 class TestBaremetalIntrospectionProxy(test_proxy_base.TestProxyBase):
 
     def setUp(self):
         super(TestBaremetalIntrospectionProxy, self).setUp()
         self.proxy = _proxy.Proxy(self.session)
 
-    def test_create_introspection(self):
-        self.verify_create(self.proxy.start_introspection,
-                           introspection.Introspection,
-                           method_kwargs={'node': 'abcd'},
-                           expected_kwargs={'id': 'abcd'})
-
     def test_get_introspection(self):
         self.verify_get(self.proxy.get_introspection,
                         introspection.Introspection)
@@ -140,3 +163,12 @@ class TestGetData(base.TestCase):
             self.proxy, 'introspection/1234/data', 'GET',
             headers=mock.ANY, microversion=mock.ANY)
         self.assertIs(data, mock_request.return_value.json.return_value)
+
+    def test_get_unprocessed_data(self, mock_request):
+        mock_request.return_value.status_code = 200
+        data = self.proxy.get_introspection_data(self.introspection,
+                                                 processed=False)
+        mock_request.assert_called_once_with(
+            self.proxy, 'introspection/1234/data/unprocessed', 'GET',
+            headers=mock.ANY, microversion='1.17')
+        self.assertIs(data, mock_request.return_value.json.return_value)
