@@ -10,7 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import testtools
+from openstack.tests.unit import base
 import uuid
 
 from openstack.network.v2 import qos_dscp_marking_rule
@@ -22,7 +22,7 @@ EXAMPLE = {
 }
 
 
-class TestQoSDSCPMarkingRule(testtools.TestCase):
+class TestQoSDSCPMarkingRule(base.TestCase):
 
     def test_basic(self):
         sot = qos_dscp_marking_rule.QoSDSCPMarkingRule()
@@ -30,10 +30,9 @@ class TestQoSDSCPMarkingRule(testtools.TestCase):
         self.assertEqual('dscp_marking_rules', sot.resources_key)
         self.assertEqual('/qos/policies/%(qos_policy_id)s/dscp_marking_rules',
                          sot.base_path)
-        self.assertEqual('network', sot.service.service_type)
         self.assertTrue(sot.allow_create)
-        self.assertTrue(sot.allow_get)
-        self.assertTrue(sot.allow_update)
+        self.assertTrue(sot.allow_fetch)
+        self.assertTrue(sot.allow_commit)
         self.assertTrue(sot.allow_delete)
         self.assertTrue(sot.allow_list)
 
